package com.xs.core.filter;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.NotPermissionException;
import cn.dev33.satoken.jwt.exception.SaJwtException;
import cn.hutool.core.exceptions.ExceptionUtil;
import com.xs.core.common.exception.BusinessException;
import com.xs.core.model.ResponseResult;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.ConstraintViolationException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.NoSuchMessageException;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.multipart.support.MissingServletRequestPartException;
import org.springframework.web.servlet.resource.NoResourceFoundException;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {
    @ExceptionHandler({Exception.class, NotLoginException.class, HttpMessageNotReadableException.class,
            HttpMediaTypeNotSupportedException.class, MaxUploadSizeExceededException.class, NotPermissionException.class,
            HttpRequestMethodNotSupportedException.class, MethodArgumentNotValidException.class, BindException.class,
            ConstraintViolationException.class, MissingServletRequestPartException.class, SaJwtException.class, NoResourceFoundException.class, BusinessException.class})
    @ResponseBody
    ResponseResult<?> serverExceptionHandler(HttpServletRequest request, HttpServletResponse response, Exception e) {
        if (e instanceof NotLoginException || e instanceof SaJwtException) {
            return ResponseResult.unauthorized(null);
        } else if (e instanceof HttpMessageNotReadableException || e instanceof HttpRequestMethodNotSupportedException) {
            return ResponseResult.badRequest(null);
        } else if (e instanceof MaxUploadSizeExceededException) {
            return ResponseResult.requestTooLarge(null);
        } else if (e instanceof HttpMediaTypeNotSupportedException) {
            return ResponseResult.unsupportedMediaType(null);
        } else if (e instanceof NotPermissionException) {
            return ResponseResult.forbidden();
        } else if (e instanceof BindException) {
            List<FieldError> fieldErrorList = ((BindException) e).getFieldErrors();
            if (!CollectionUtils.isEmpty(fieldErrorList)) {
                for (FieldError fieldError : fieldErrorList) {
                    if (fieldError != null && fieldError.getDefaultMessage() != null) {
                        String field = fieldError.getField();
                        String defaultMessage = fieldError.getDefaultMessage();
                        // 尝试从消息源获取国际化消息，如果没有找到，则使用默认消息
                        return ResponseResult.failed(defaultMessage);
                    }
                }
            }
            return ResponseResult.failed("response.validation.failed", null);
        } else if (e instanceof ConstraintViolationException) {
            return ResponseResult.badRequest(e.getMessage());
        } else if (e instanceof BusinessException) {
            //增加自定义内容响应
            return ResponseResult.failed(e.getMessage());
        } else if (e instanceof MissingServletRequestPartException) {
            return ResponseResult.failed("response.expected.file.missing", null);
        } else if (e instanceof NoResourceFoundException) {
            return ResponseResult.failed("request.api.not.exists", null);
        } else {
            log.error("请求:{} 异常,异常信息:{}", request.getRequestURI(), ExceptionUtil.stacktraceToString(e));
            return ResponseResult.serverError(null);
        }
    }
}
