package com.xs.core.mq.producer;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson2.JSON;
import com.xs.core.config.mq.RabbitMQConfig;
import com.xs.core.model.coin.msg.CoinProducerMessage;
import com.xs.core.model.coin.msg.RetryableMessage;
import com.xs.core.model.coin.msg.TeamShareMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.MessageDeliveryMode;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.function.*;

@Component
@Slf4j
public class GoldCoinMessageProducer {
    @Autowired
    private RabbitTemplate rabbitTemplate;

    private static final int MAX_RETRY_COUNT = 3;
    private static final int BASE_DELAY = 1000; // 1 second

    /**
     * 发送计算金币产出的消息 loop
     */
    public void sendDelayCalculationMessage(CoinProducerMessage coinProducerMessage) {
        sendMessageWithRetry(coinProducerMessage, MAX_RETRY_COUNT, m -> {
            String jsonString = JSON.toJSONString(m);
            rabbitTemplate.convertAndSend(RabbitMQConfig.GOLD_COIN_CALCULATION_EXCHANGE, RabbitMQConfig.GOLD_COIN_CALCULATION_QUEUE, jsonString, msg -> {
                //延时10秒
                msg.getMessageProperties().setHeader("x-delay", 10 * 1000L);
                return msg;
            });
        });
    }

    /**
     * 发送计算完成的消息
     *
     * @param coinProducerMessage 计算完成的消息
     */
    public void sendCalculationCompleteMessage(CoinProducerMessage coinProducerMessage) {
        sendMessageWithRetry(coinProducerMessage, MAX_RETRY_COUNT, m -> {
            rabbitTemplate.convertAndSend(RabbitMQConfig.GOLD_COIN_COMPLETE_EXCHANGE, RabbitMQConfig.GOLD_COIN_COMPLETE_QUEUE, JSON.toJSONString(m));
            log.info("发送计算结束的消息: {}", JSON.toJSONString(m));
        });
    }

    /**
     * 发送团队奖励的消息
     *
     * @param teamShareMessage
     */
    public void sendTeamShareMessage(TeamShareMessage teamShareMessage) {
        sendMessageWithRetry(teamShareMessage, MAX_RETRY_COUNT, m -> rabbitTemplate.convertAndSend(RabbitMQConfig.GOLD_COIN_TEAM_SHARE_EXCHANGE, RabbitMQConfig.GOLD_COIN_TEAM_SHARE_QUEUE, JSON.toJSONString(m), msg -> {
            //延时5秒
            msg.getMessageProperties().setHeader("x-delay", 5 * 1000L);
            return msg;
        }));
    }

    /**
     * 通用的消息发送方法
     *
     * @param message    要发送的消息
     * @param maxRetries 最大重试次数
     * @param sendAction 发送消息的动作
     * @param <T>        消息类型
     */
    public <T> void sendMessageWithRetry(T message, int maxRetries, Consumer<T> sendAction) {
        try {
            sendAction.accept(message);
        } catch (Exception e) {
            // 处理发送失败的逻辑
//            log.error("Failed to send message: {}", message, e);
            //handleSendFailure(message, maxRetries, sendAction);
        }
    }

    /**
     * 处理发送失败的通用方法
     */
    private <T> void handleSendFailure(T message, int maxRetries, Consumer<T> sendAction) {
        if (message instanceof RetryableMessage retryableMessage && ((RetryableMessage) message).getRetryCount() < maxRetries) {
            retryableMessage.setRetryCount(retryableMessage.getRetryCount() + 1);

            // 使用指数退避策略
            int delay = (int) Math.pow(2, retryableMessage.getRetryCount()) * BASE_DELAY;
            CompletableFuture.delayedExecutor(delay, TimeUnit.MILLISECONDS).execute(() -> sendMessageWithRetry((T) retryableMessage, maxRetries, sendAction));
        } else {
            // 记录失败日志，可以考虑报警
            log.error("Failed to send message after {} retries: {}", maxRetries, message);
        }
    }
}
