package com.xs.core.service.Impl.user;

import cn.dev33.satoken.stp.SaLoginConfig;
import cn.dev33.satoken.stp.SaTokenInfo;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.xs.core.common.constant.ConstantConfig;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.enums.DisEnableStatusEnum;
import com.xs.core.common.validation.CheckUtils;
import com.xs.core.model.user.entity.User;
import com.xs.core.model.user.req.AppLoginReq;
import com.xs.core.model.wallet.entity.UserWallet;
import com.xs.core.service.team.TeamShareService;
import com.xs.core.service.user.AppLoginService;
import com.xs.core.service.user.IUserService;
import com.xs.core.service.wallet.IUserWalletService;
import com.xs.core.utils.InviteCodeGenerator;
import com.xs.core.utils.SpringWebUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.concurrent.CompletableFuture;

@Service
@AllArgsConstructor
@Slf4j
public class AppLoginServiceImpl implements AppLoginService {
    private final IUserService userService;

    private final TeamShareService teamShareService;

    private final IUserWalletService walletService;

    @Override
    public SaTokenInfo tgLogin(AppLoginReq loginReq) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getTgId, loginReq.getId());
        User user = userService.getOne(queryWrapper);
        if (ObjUtil.isNull(user)) {
            user = registerToSys(loginReq);
        }
        //校验是否被禁用
        checkUserStatus(user);
        return this.login(user);
    }

    @Override
    @Transactional
    public User registerToSys(AppLoginReq loginReq) {
        User user = User.builder()
                .id(IdWorker.getId())
                .tgAccount(loginReq.getUsername())
                .tgId(loginReq.getId())
                .avatar(StrUtil.isNotBlank(loginReq.getPhotoUrl()) ? URLDecoder.decode(loginReq.getPhotoUrl(), StandardCharsets.UTF_8) : "")
                .firstName(loginReq.getFirstName())
                .lastName(loginReq.getLastName())
                //注册时指定用户的唯一邀请码
                .inviteCode(InviteCodeGenerator.generateInviteCode())
                .languageCode(loginReq.getLanguageCode())
                .createdTime(LocalDateTime.now())
                .updatedTime(LocalDateTime.now())
                .build();
        if (StrUtil.isNotBlank(loginReq.getAge_limit())) {
            user.setAgeLimit(Integer.parseInt(loginReq.getAge_limit()));
        }
        User inviteUser;
        if (StrUtil.isNotBlank(loginReq.getShare_code())) {
            //如邀请码不为空 则写入邀请人信息
            LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(User::getInviteCode, loginReq.getShare_code());
            inviteUser = userService.getOne(queryWrapper);
            if (ObjUtil.isNotNull(inviteUser)) {
                user.setReferrerId(String.valueOf(inviteUser.getId()));
                user.setPassengerFlowWay(2);
                user.setId(IdWorker.getId());
            }
        } else {
            inviteUser = null;
        }
        //初始化钱包信息
        UserWallet userWallet = new UserWallet();
        userWallet.setUserId(user.getId());
        userWallet.setUpdatedTime(LocalDateTime.now());
        userService.save(user);
        walletService.save(userWallet);
        CompletableFuture.runAsync(() -> {
            //注册成功后发放邀请奖励
            if (ObjUtil.isNotNull(inviteUser)) {
                teamShareService.teamInviteGoldCoinRewardGeneral(inviteUser.getId(), user.getId());
            }
        });
        return user;
    }

    /**
     * 检查用户状态
     *
     * @param user 用户信息
     */
    private void checkUserStatus(User user) {
        CheckUtils.throwIfEqual(DisEnableStatusEnum.DISABLE.getValue(), user.getDisableFlag(), "response.user.disable");
    }

    private SaTokenInfo login(User user) {
        Long userId = user.getId();
        user.setLoginTime(LocalDateTime.now());
        UserContext userContext = new UserContext();
        BeanUtils.copyProperties(user, userContext);
        userContext.setId(userId);
        // 记录登录时间以及ip等信息
        userContext.setExtraInfo(SpringWebUtils.getRequest());
        user.setLoginIp(userContext.getIp());
        user.setIpAddressConvert(userContext.getAddress());
        //先登出 防止同一个账号重复登录生成多个token
        StpUtil.logout(userId);
        // 登录并缓存用户信息
        StpUtil.login(userId, SaLoginConfig.setExtra(ConstantConfig.USER_CONTENT_EXTRA_KEY, JSON.toJSONString(userContext)));
        CompletableFuture.runAsync(() -> userService.updateById(user));
        return StpUtil.getTokenInfo();
    }
}
